<?php



use Sugarcrm\Sugarcrm\Security\HttpClient\ExternalResourceClient;
use Sugarcrm\Sugarcrm\Security\HttpClient\RequestException;

class RestHelper_lb6606d3904e3114_10412513
{
    public static function call(
        $url,
        $type = 'GET',
        $headers = [],
        $arguments = [],
        $proxy = null,
        $httpAuth = null
    )
    {
        $method = strtoupper($type);
        $url = self::formatUrl($url, $method, $arguments);
        $requestBody = self::getRequestBody($method, $arguments);
        $headers = self::composeHeaders($headers, $httpAuth);
        $response = self::request($method, $url, $requestBody, $headers);
        if (empty($response)) {
            return [];
        }
        return [
            'result' => $response->getBody()->getContents(),
            'info' => [
                'http_code' => $response->getStatusCode(),
                'content_type' => self::getResponseHeader($response, 'Content-Type'),
            ],
        ];
    }

    private static function getResponseHeader($response, $headerName)
    {
        $responseHeaders = $response->getHeaders();
        if (!isset($responseHeaders[$headerName])) {
            return null;
        }
        $value = $responseHeaders[$headerName];
        if (!is_array($value)) {
            return $value;
        }
        return $value[0];
    }

    private static function request($method, $url, $body, $headers)
    {
        $client = new ExternalResourceClient(60, 10);
        try {
            return $client->request(
                !empty($method) ? $method : 'GET',
                !empty($url) ? $url : '',
                $body,
                !empty($headers) ? $headers : []
            );
        } catch (RequestException $e) {
            return null;
        }
    }

    private static function formatUrl($url, $method, $arguments)
    {
        if (empty($arguments)) {
            return $url;
        }
        if ($method != 'GET') {
            return $url;
        }
        $connector = (strpos($url, '?')) ? '&' : '?';
        return $url . $connector . http_build_query($arguments);
    }

    private static function getRequestBody($method, $arguments)
    {
        if ($method == 'GET') {
            return null;
        }
        if (is_string($arguments)) {
            return $arguments;
        }
        return json_encode($arguments);
    }

    private static function composeHeaders($headers, $httpAuth)
    {
        if (self::isBasicAuth($httpAuth)) {
            $headers['Authorization'] = self::formatBasicAuthHeader($httpAuth['username'], $httpAuth['password']);
        }
        return $headers;
    }

    private static function isBasicAuth($httpAuth)
    {
        if (!is_array($httpAuth)) {
            return false;
        }
        if (!isset($httpAuth['username']) || !isset($httpAuth['password'])) {
            return false;
        }
        return true;
    }

    private static function formatBasicAuthHeader($username, $password)
    {
        return 'Basic ' . base64_encode("{$username}:{$password}");
    }
}

